-- CO Mission Enabler UI
-- Author: sman
-- DateCreated: 02/16/18 4:11:16 AM
--------------------------------------------------------------

include( "IconSupport" )
include( "InstanceManager" )
include( "CommonBehaviors" )
include( "GameplayUtilities" )

include( "GovernmentOverviewAIFunctions.lua" )
include( "GovernmentOverviewFunctions.lua" )
include( "GovernmentOverviewManageSaveTable.lua" )
include( "GovernmentOverviewSelectPopup.lua" )

-- ========================================================================================================================
--		Globals

include( "GovernmentOverviewGlobals.lua" )

ContextPtr:SetHide(true)


-- =========================================================================================================================================
--		Input Handlers
-- =========================================================================================================================================

function On_GOV_OV_CLOSE()																-- Update enabled mission list, based on current status of COME popup
	ContextPtr:SetHide(true)																-- Hide popup
	UIManager:DequeuePopup( ContextPtr );	
end
Controls.CloseButton:RegisterCallback(Mouse.eLClick, On_GOV_OV_CLOSE)


function GOV_OV_InputHandler( uiMsg, wParam, lParam )
    if uiMsg == KeyEvents.KeyDown then
        if wParam == Keys.VK_ESCAPE or wParam == Keys.VK_RETURN then			
			On_GOV_OV_CLOSE()			
            return true
        end
    end 
end
ContextPtr:SetInputHandler( GOV_OV_InputHandler )


-- =========================================================================================================================================
-- Close Up Panel - lower half of the UI
-- =========================================================================================================================================

function PopulateCloseUpPanel(tMember)														-- Populate the Close Up (lower) Panel
	local pPlayer = Players[tMember.PlayerId]
		
	-- Populate Header/Border data
	local iCivDescernment = tMember.CivDiscernment or CD_UNKNOWN
	dPrint("Target Civ: " .. pPlayer:GetCivilizationShortDescription() .. "     Civ Discernment: " .. g_tCiv_Discernment[iCivDescernment][1])

	local sCivName = pPlayer:GetCivilizationDescription()
	local sLeaderName = GameplayUtilities.GetLocalizedLeaderTitle(pPlayer) or "Unknown"	
	local iGovernmentType = GetGovernmentType(pPlayer)
	local sGovernmentType = g_tGovernment_Types[iGovernmentType][1] or "Unknown"
	local iGovernmentDisplayIcon = iGovernmentType

	if not tMember.HasMet then
		sCivName = "Unknown Empire"
		sLeaderName = "Unknown Player"
		sGovernmentType = "Unknown"		
		iGovernmentType = 16
		iGovernmentDisplayIcon = 0
	end	

	Controls.CloseUpPanelHeader:SetHide(false)												-- Display the Leader name, and gov icons
	Controls.CloseUpLeaderTitle:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_CLOSEUP_TITLE", sCivName, sLeaderName) )

	IconHookup(tMember.PortraitIndex, 128, tMember.IconAtlas, Controls.CloseUpLeaderIcon);	-- Fill in the Leader Icon on the LHS
	Controls.CloseUpLeaderIcon:SetToolTipString(Locale.ConvertTextKey("TXT_KEY_GOV_OV_CLOSEUP_LEADER_TT", sLeaderName));

	IconHookup(tMember.CivPortraitIndex, 128, tMember.CivIconAtlas, Controls.CloseUpCivIcon);	-- Fill in the Civ Icon on the LHS
	Controls.CloseUpCivIcon:SetToolTipString(Locale.ConvertTextKey("TXT_KEY_GOV_OV_CLOSEUP_CIV_TT", sCivName));
	
	IconHookup(iGovernmentDisplayIcon, 128, "SMAN_NATIONAL_GOVERNMENTS_ATLAS", Controls.CloseUpGovIcon);		-- Fill in the Gov Type on the LHS	
	Controls.CloseUpGovIcon:SetToolTipString(Locale.ConvertTextKey("TXT_KEY_GOV_OV_CLOSEUP_GOV_TT", sGovernmentType, Locale.ConvertTextKey(g_tGovernment_Types[iGovernmentType][4])));
		
	if tMember.HasMet then
		if (iCivDescernment == GOV_CD_INTRODUCTORY) or (iCivDescernment == GOV_CD_BASIC) or (iCivDescernment == GOV_CD_FUNCTIONAL) then		-- If the Civ Disc is "has met", "has embassy", or "has spy in non-capital city", then...
			PopulateLowerPanelforLessKnownCiv(pPlayer, sGovernmentType, tMember.CivDiscernment)		-- Use "assessment" data, instead of adjusted numbers
		else																						-- Remaining options: Advanced, Fluent, and Native (detailed report)
			PopulateLowerPanelforWellKnownCiv(pPlayer, sGovernmentType, tMember.CivDiscernment)		-- Use adjusted numbers, not verbiage
		end
	else
		PopulateLowerPanelforUnknownCiv()
	end
				
	--	Enable (for human player) / Disable (for AI player) "Change Government" button	
	local iGovStartTurn = g_tSavedData.GetValue(string.format("GOV_OV_GOV_AGE_PLAYER_%02i", pPlayer:GetID() ) ) 
	if not iGovStartTurn then iGovStartTurn = Game.GetStartTurn() end						-- Get game turn old govt started, but use the game's starting turn if nothing found
	local iGovAge = Game.GetGameTurn() - iGovStartTurn	
	dPrint("Reading Gov Age. Player: " .. pPlayer:GetCivilizationShortDescription() .. "     Gov Started on GT: " .. iGovStartTurn  .. "    Curent Gov Age: " .. iGovAge)
	
	local iGoldCost = GetGovernmentChangeCost(pPlayer)
	Controls.CloseUpChangeGovernment:SetToolTipString(Locale.ConvertTextKey("TXT_KEY_GOV_OV_CLOSEUP_CHANGE_GOV_TT", iGoldCost))

	if ( pPlayer:IsHuman() ) and ( pPlayer:GetNumCities() > 0 ) and ( iGovAge > g_MinimumGovernmentAge ) and		-- Account for "complete kills" - you can't choose a govt if you don't have a city to house the gov building																				
				(pPlayer:GetGold() >= iGoldCost) then														-- pPlayer must have sufficient gold to change govt	
		Controls.CloseUpChangeGovernment:SetDisabled(false)													-- Enable the launch button
		Controls.CloseUpChangeGovernment:RegisterCallback( Mouse.eLClick, ChangeActivePlayerGovernment )	-- Enable the callback to Change Govt function
	elseif ( pPlayer:IsHuman() ) then
		Controls.CloseUpChangeGovernment:SetDisabled(true)													-- Disable the launch button
	else
		Controls.CloseUpChangeGovernment:SetToolTipString(TXT_KEY_GOV_OV_CLOSEUP_CHANGE_GOV_OTHER_PLAYER_TT)	-- Change the TT for non-human player
		Controls.CloseUpChangeGovernment:SetDisabled(true)													-- Disable the launch button
	end	
end


function OnLeaderSelected(tMember)															-- On click on top-panel portrait, then...
	if (not Players[Game.GetActivePlayer()]:IsTurnActive() or Game.IsProcessingMessages()) then
		return;
	end

	PopulateCloseUpPanel(tMember)

	if (Game.GetActivePlayer() == ePlayer) then	
		return;
	end
    --if (Players[ePlayer] ~= nil) then	
	--end
end


-- =========================================================================================================================================
-- World Governments Panel - upper half of the UI
-- =========================================================================================================================================

function GetWorldGovernments()																-- Get government data for all major civs; populate tMembers table
	dPrint("GetWorldGovernments")
	
	local tMembers = {};
	local iHostPlayerId = 0
	local iHostTeamID = Players[iHostPlayerId]:GetTeam()
	
	for iPlayer = 0, GameDefines.MAX_MAJOR_CIVS - 1, 1 do
		local pPlayer = Players[iPlayer];
		if ( pPlayer ~= nil and pPlayer:IsAlive() ) then	
									
			local tMember = {};			
			local pLeader = GameInfo.Leaders[pPlayer:GetLeaderType()];				
			tMember.PlayerId = iPlayer;					

			if (Teams[iHostTeamID]:IsHasMet(pPlayer:GetTeam())) or (iPlayer == iHostPlayerId) then				
				tMember.HasMet = true
				tMember.PortraitIndex = pLeader.PortraitIndex;
				tMember.IconAtlas = pLeader.IconAtlas;	
				tMember.Ranking = "?"				
				tMember.CivPortraitIndex = GameInfo.Civilizations[ Players[iPlayer]:GetCivilizationType() ].PortraitIndex
				tMember.CivIconAtlas = GameInfo.Civilizations[ Players[iPlayer]:GetCivilizationType() ].IconAtlas
				--tMember.ShowSpy = Players[iHostPlayerId]:IsMyDiplomatVisitingThem(iPlayer)		-- Need this to dipaly Diplo icon under Leader portrait
				tMember.CivDiscernment = GetCivilizationalDiscernment(pPlayer) or CD_UNKNOWN
			else
				tMember.HasMet = false
				tMember.PortraitIndex = 23
				tMember.IconAtlas = "CIV_COLOR_ATLAS"	
				tMember.Ranking = "?"
				tMember.CivPortraitIndex = 23
				tMember.CivIconAtlas = "CIV_COLOR_ATLAS"
				--tMember.ShowSpy = Players[iHostPlayerId]:IsMyDiplomatVisitingThem(iPlayer)		-- Need this to dipaly Diplo icon under Leader portrait
				tMember.CivDiscernment = GetCivilizationalDiscernment(pPlayer) or CD_UNKNOWN
			end

			if(iPlayer == iHostPlayerId) then
				table.insert(tMembers, 1, tMember);
			else
				table.insert(tMembers, tMember);
			end
		end		
	end	
	return tMembers;
end


function ViewWorldGovernments(members)
	local maxMemberControls = 21;	
	for i = 1, maxMemberControls, 1 do
		local buttonControlID = "MemberButton" .. i;
		local control = Controls[buttonControlID];
		if(control ~= nil) then
			control:SetHide(true);
		end
	end
	
	IconHookup(15, 64, "SMAN_NATIONAL_GOVERNMENTS_ATLAS", Controls.TitleIcon)			-- Initial center icon will be "?"
	ContextPtr:SetHide(false)
	Controls.GovernmentOverviewContent:SetHide(false);
	
	for i, member in ipairs(members) do
		local buttonControl;	local iconSize;		local iconControl;		local votesControl;		local scrollControl;		local spyControl;
		local subIconSize;		local subIcon;		local subIconBG;		local subIconShadow;	local subIconHighlight;
		--dPrint("i: " .. i .. "     Player: " .. member.PlayerId .. "     Atlas: " .. member.IconAtlas .. "     PortraitIDX: " .. member.PortraitIndex)
				
		if(i == 1) then																		-- Assume the first member is the leader.
			buttonControl = Controls.LeaderButton;
			iconControl = Controls.LeaderIcon;
			iconSize = 128;
			votesControl = Controls.LeaderVotes;
			scrollControl = Controls.LeaderScroll;
			spyControl = Controls.LeaderSpy;
			
			subIconSize = 45;
			subIcon = Controls.LeaderSubIcon;
			subIconBG = Controls.LeaderSubIconBG;
			subIconShadow = Controls.LeaderSubIconShadow;
			subIconHighlight = Controls.LeaderSubIconHighlight;

			PopulateCloseUpPanel(member)
			
		else
			local index = tostring(i - 1);
			buttonControl = Controls["MemberButton" .. index];
			iconControl = Controls["MemberIcon" .. index];
			iconSize = 64;
			votesControl = Controls["MemberVotes" .. index];
			scrollControl = Controls["MemberScroll" .. index];
			spyControl = Controls["MemberSpy".. index];
			
			subIconSize = 32;
			subIcon = Controls["MemberSubIcon" .. index];
			subIconBG = Controls["MemberSubIconBG" .. index];
			subIconShadow = Controls["MemberSubIconShadow" .. index];
			subIconHighlight = Controls["MemberSubIconHighlight" .. index];
		end
	
		buttonControl:SetHide(false);
		IconHookup(member.PortraitIndex, iconSize, member.IconAtlas, iconControl);      -- Attach Leader icon (size 64)  
		
		local iMiniID = member.PlayerId	
		local sPlayerCivName = 	Players[member.PlayerId]:GetCivilizationDescription()

		if not member.HasMet then
			iMiniID = -1	
			sPlayerCivName = "Unknown Empire"		
		end		
		
		CivIconHookup(iMiniID, subIconSize, subIcon, subIconBG, subIconShadow, true, true, subIconHighlight);	-- Attach Civ icon (size 32)
		local iPlayerRank, iActivePlayers = GetRank( STAT_SCORE, member.PlayerId )
		votesControl:SetText( tostring( iPlayerRank ) )
		local sPlayerRank = GetOrdinalNumber(iPlayerRank) or "??"
		local sLeaderButtonTT = sPlayerCivName .. " is in " .. sPlayerRank .. " place"

		if i ~= 1 then
			sLeaderButtonTT = sLeaderButtonTT .. "[NEWLINE][NEWLINE]" .. g_tCiv_Discernment[member.CivDiscernment][2]
		end
		
		sLeaderButtonTT = sLeaderButtonTT .. "[NEWLINE][NEWLINE][COLOR_POSITIVE_TEXT]LEFT-CLICK[ENDCOLOR] on the Leader Icon to see a detailed report on this Civilization in the lower panel."
		buttonControl:SetToolTipString(sLeaderButtonTT);
		
		scrollControl:SetHide(true);
				
		if (member.CivDiscernment == GOV_CD_FUNCTIONAL) or (member.CivDiscernment == GOV_CD_ADVANCED) then	-- If active player has a spy in non-Cpital cities or in the capital, then...
			spyControl:SetText( "[ICON_SPY]" )
			spyControl:SetHide(false)
		elseif (member.CivDiscernment == GOV_CD_FLUENT) then								-- If the active player has a diplomat, then...
			spyControl:SetText( "[ICON_DIPLOMAT]" )
			spyControl:SetHide(false)
		else
			spyControl:SetHide(true)
		end
				
		buttonControl:RegisterCallback( Mouse.eLClick, function() OnLeaderSelected(member); end );		

	end
end


function WAW_GOV_Popup()																	-- Unhides the Governments Popup, after selected from Additional Info menu
	local tMembers = GetWorldGovernments()
	
	BuildStatsTable()
	ViewWorldGovernments(tMembers)
end


function WAW_GOV_UpdateAdditionInformationMenu(additionalEntries)							-- Add a menu item to the Additional Information menu
	table.insert(additionalEntries, {
		text="World Governments", 
		call=WAW_GOV_Popup
	})
end
LuaEvents.AdditionalInformationDropdownGatherEntries.Add( WAW_GOV_UpdateAdditionInformationMenu )
LuaEvents.RequestRefreshAdditionalInformationDropdownEntries()





