-- GovernmentOverviewAIFunctions.lua
-- Author: sman
-- DateCreated: 08/03/18 6:22:22 AM
--------------------------------------------------------------

function GetQuartile (iStat, iPlayerID)														-- Simple quartile forumla based on rank vs. number of active players; returns 1..4
	local iQuartile = 4
	local iRank, iActivePlayers = GetRank( iStat, iPlayerID )
	
	if iRank <= (iActivePlayers / 4) then
		iQuartile = 1
	elseif iRank <= (iActivePlayers / 2) then
		iQuartile = 2
	elseif iRank <= (iActivePlayers / 1.3333) then
		iQuartile = 3
	end
	--dPrint("Stat: " .. iStat .. "    TotalPlayer: " .. iActivePlayers .. "     Rank: " .. iRank .. "     Quartile: " .. iQuartile)
	return iQuartile
end


function CalculateAIGovernmentChangePressure(pPlayer)										-- Determines how much appetite for gov change pPlayer has
	local iPlayerID = pPlayer:GetID()
	local iChangePressure = 0

	BuildStatsTable()

	local iCurrentHappiness = pPlayer:GetExcessHappiness() or 0								-- Excess Happiness
	local iHappinessPressure = 0
	if iCurrentHappiness > 0 then								
		iHappinessPressure = iCurrentHappiness * 2
	else
		iHappinessPressure = iCurrentHappiness
	end

	local iGoldPerTurn = pPlayer:CalculateGoldRate() or 0									-- Gold per Turn pressure
	local iGoldPressure = 0
	if iGoldPerTurn > 0 then																
		local iTempGold = math.floor ( iGoldPerTurn / 20 )									-- Divide GPT / 20 (rounded down)
		if iTempGold > 20 then iTempGold = 20 end											-- Limit of +20 (which would mean +400 GPT or more)
		iGoldPressure = iTempGold
	else
		local iTempGold = math.floor ( iGoldPerTurn / 20 )									-- Divide GPT / 20 (rounded down)
		if iTempGold < -20 then iTempGold = -20 end											-- Limit of -20 (which would mean -400 GPT or more)
		iGoldPressure = iTempGold 										
	end

	local iTotalGold = pPlayer:GetGold() or 0												-- Total Gold pressure
	local iTotalGoldPressure = 0	
	iTempGold = math.abs( math.floor ( iTotalGold / 100 ) )									-- Divide Total Gold / 100 (rounded down)
	if iTempGold > 20 then iTempGold = 20 end												-- Limit of +20 (which would mean +2,000 Gold or more)
	iTotalGoldPressure = iTempGold
	
	local iScorePressure = ( 3 - GetQuartile (STAT_SCORE, iPlayerID) ) * 3					-- Misc. stat pressures
	local iPopulationPressure = ( 3 - GetQuartile (STAT_POPULATION, iPlayerID) ) * 3		-- Adjust quartile - 2, so the range = -2 .. +2, then multiply by +2; range is -4 .. +4
	local iMilPowerPressure = ( 3 - GetQuartile (STAT_MIL_POWER, iPlayerID) ) * 3
	local iLiteracyPressure = ( 3 - GetQuartile (STAT_LITERACY, iPlayerID) ) * 3
	--print("Quartiles. Score: " .. iScoreQuartile .. "     Pop: " .. iPopulationQuartile .. "     Mil: " .. iMilPowerQuartile .. "     Lit: " .. iLiteracyQuartile)

	local iWonderPressure = pPlayer:GetNumWorldWonders() * 1.5 or 0							-- Wonder pressure (all positive)

	local iCapitalTally = 0																	-- Number of total Capitals in empire (all positive)
	for pCity in pPlayer:Cities()do 				
		if pCity:IsOriginalCapital() then iCapitalTally = iCapitalTally  + 1 end
	end	
	local iCapitalPressure = iCapitalTally * 2

	local iGovStartTurn = g_tSavedData.GetValue(string.format("GOV_OV_GOV_AGE_PLAYER_%02i", iPlayerID ) )	-- Age of Government pressure (all negative or 0)
	if not iGovStartTurn then iGovStartTurn = Game.GetStartTurn() end						-- Get game turn old govt started, but use the game's starting turn if nothing found
	local iGovAge = Game.GetGameTurn() - iGovStartTurn
	iGovAgePressure = math.floor( (iGovAge / g_MinimumGovernmentAge) + .5) * -1				-- Number of min turn periods current gov has existed - more pressure the longer the gov is in existence
	if iGovAgePressure < -12 then iGovAgePressure = -12 end									-- Limit to how much gov age impacts change pressure
	
	iChangePressure = math.floor( iHappinessPressure + iTotalGoldPressure + iGoldPressure + iScorePressure + iPopulationPressure + iMilPowerPressure + iLiteracyPressure + iWonderPressure + iCapitalPressure + iGovAgePressure )
	print("Pressures. Happiness: " .. iHappinessPressure .. "    Gold: " .. iTotalGoldPressure .. "    Gold per Turn: " .. iGoldPressure .. "    Score: " .. iScorePressure .. "    Population: " .. iPopulationPressure .. "    Mil Power: " .. iMilPowerPressure .. "   Literacy: " .. iLiteracyPressure .. "    Wonder: " .. iWonderPressure .. "    Capitals: " .. iCapitalPressure .. "    Age: " .. iGovAgePressure)
	print("Final Pressure: " .. iChangePressure .. " - note: lower numbers mean MORE pressure to change")

	return iChangePressure

end -- =============================================================================		CalculateAIGovernmentChangePressure


function ChangeAIGovernment(pPlayer)														-- Function will change pPlayer's gov to the latest one available to them
	g_ActivePlayer = pPlayer
	
	local iCurrentGold = pPlayer:GetGold() or 0
	local iGoldCost = GetGovernmentChangeCost(pPlayer)	
	if (iGoldCost > iCurrentGold) and (math.random(1,100) < 50 ) then						-- if Gold cost is more than pPlayer has AND a random 50% check, then exit early; if they have enough or the random test fails, gov WILL change
		return
	end

	local iCurrentGovType = GetGovernmentType(pPlayer)
	local iNewGovType = GOV_ANARCHY	
	local iIdeology, sIdeology = GetIdeology(pPlayer)
	local iCurrentCulture = pPlayer:GetJONSCulture() or 0
		
	local pCapitalCity = pPlayer:GetCapitalCity()
	if not pCapitalCity then return end														-- If AI civ has no capital, exit early
		
	for iDX = GOV_TECHNOCRACY, GOV_DESPOTISM, -1 do											-- Scroll through all Governments from latest to earliest
		if IsValidGovernment(sIdeology, iDX) and CanPlayerBuildGovernment(pPlayer, iCurrentGovType, iDX) then
			iNewGovType = iDX
			break
		end
	end
	
	if iCurrentGovType ~= iNewGovType then													-- There really has been a change in gov, so...
		dPrint("AI Gov Change.  Civ: " .. pPlayer:GetCivilizationAdjective() .. "   Old Gov: " .. g_tGovernment_Types[iCurrentGovType][1] .. "      New Gov: " .. g_tGovernment_Types[iNewGovType][1] )
		
		ClearGovernmentBuildings(pPlayer)													-- Remove all gov buildings from pPlayer's capital
		local iNewGovBuildingType = GameInfoTypes[ g_tGovernment_Types[iNewGovType][3] ]
		pCapitalCity:SetNumRealBuilding(iNewGovBuildingType, 1)								-- Adds new govt building building type from city	
				
		g_tSavedData.SetValue(string.format("GOV_OV_GOV_AGE_PLAYER_%02i", pPlayer:GetID()), Game.GetGameTurn())	-- update Save Data table with this GT for this civ
		dPrint("Saving Gov Age. Player: " .. pPlayer:GetCivilizationShortDescription() .. "     GT: " .. Game.GetGameTurn() )
		
		local iAnarchyDuration = PenalizGovernmentForChanging(pPlayer, iGoldCost)			-- Impose "costs" for gov change; start Anarchy in pPlayer's civ; duration based on Era/Difficulty/Game Speed
		RemovePromotionFromOldGovernment(pPlayer, iNewGovType)								-- Remove the old gov's associated Promotion
		
		if Players[0]:IsMyDiplomatVisitingThem(pPlayer:GetID()) then						-- If human player has diplomats at pPlayer city, then send notification to human player that gov is changing
			SentNotification(2, pPlayer:GetCivilizationShortDescription() .. " has changed its government from " .. g_tGovernment_Types[iCurrentGovType][1] .. " to " .. g_tGovernment_Types[iNewGovType][1] .. "!",
				"The " .. pPlayer:GetCivilizationAdjective() .. " Empire has a new government!", pCapitalCity:GetX(), pCapitalCity:GetY() )
		end
	end
end


-- ========================================================================================================================
--		AI Change Government Main Function

function AIGovChangeProcess(iPlayerID)
	local pPlayer = Players[iPlayerID]

	if (pPlayer:IsHuman()) or (pPlayer:IsMinorCiv()) or (pPlayer:IsBarbarian()) then return end	-- Only process major AI civs, all others return early
	
	local pCapitalCity = pPlayer:GetCapitalCity()											-- If AI civ has no capital, exit early (for complete kills option)
	if not pCapitalCity then 
		dPrint("AIGovChangeProcess. Player: " .. pPlayer:GetCivilizationShortDescription() .. " has no capital city. Exiting early...")
		return 
	end														

	-- Check to make sure the gov is old enough to consider changing 
	local iGovStartTurn = g_tSavedData.GetValue(string.format("GOV_OV_GOV_AGE_PLAYER_%02i", pPlayer:GetID() ) ) 
	if not iGovStartTurn then iGovStartTurn = Game.GetStartTurn() end						-- Get game turn old govt started, but use the game's starting turn if nothing found
	local iGovAge = Game.GetGameTurn() - iGovStartTurn	
		
	local iCurrentGovType = GetGovernmentType(pPlayer)
	dPrint("===================================================")
	dPrint("Processing AI Change Gov turn. Civ: " .. pPlayer:GetCivilizationShortDescription() .. "     GT: " .. Game.GetGameTurn() .. "    Gov: " .. g_tGovernment_Types[iCurrentGovType][1] .. "    Gov Age: " .. iGovAge)

	if iGovAge < g_MinimumGovernmentAge then												-- Current gov is too young to change; exit early
		dPrint("Gov too young to change.  Age: " .. iGovAge)
		return
	end

	local iChangePressure = CalculateAIGovernmentChangePressure(pPlayer)
	dPrint("Final change pressure: " .. iChangePressure)
	
	local iGovChangeChance = 75 - iChangePressure											-- Determine the chance the gov will change this turn
	if iGovChangeChance > 33 then iGovChangeChance = 33 end									-- If the change is more than 33%, cap it there
	
	if math.random(1, 100) < iGovChangeChance then											-- If a random number is less than the change chance, then...  (CHANCE GOV)
		ChangeAIGovernment(pPlayer)															-- Function will change pPlayer's gov to the latest one available to them
	end

end -- =============================================================================		AIGovChangeProcess
GameEvents.PlayerDoTurn.Add( AIGovChangeProcess )	





