-- GovernmentOverviewFunctions.lua
-- Author: sman
-- DateCreated: 07/29/18 1:24:45 PM
--------------------------------------------------------------

function dPrint(sPrintString)
	if g_bDebugPrint then print(sPrintString) end
end


-- ========================================================================================================================
--		Populate Lower Panel Functions - for Less, Well, and Un-known Civs

function PopulateLowerPanelforLessKnownCiv(pPlayer, sGovernmentType, iCivDiscernment )		-- For Introductory (3), Basic (4), and Functional (5) Civ Discernments
	Controls.CloseUpPanelCivData:SetHide(false)
	Controls.CloseUpPanelUnknownCivData:SetHide(true)

	local ipPlayerID = pPlayer:GetID()
	local pActivePlayer = Players[Game.GetActivePlayer()]									-- Get the Active Player pointer
	local iGetCivilizationalDiscernmentAdjustment = GetCivilizationalDiscernmentAdjustment(pPlayer) or 0
	local iWorkingCivDiscernment = iCivDiscernment + iGetCivilizationalDiscernmentAdjustment
	dPrint("PLPLessKC. CD: " .. iCivDiscernment .. "     Adjustment: " .. iGetCivilizationalDiscernmentAdjustment .. "     Working CD: " .. iWorkingCivDiscernment)
	if (iWorkingCivDiscernment >= GOV_CD_FUNCTIONAL) then iWorkingCivDiscernment = GOV_CD_FUNCTIONAL end		-- Can never adjust to ADVANCED, FLUENT or NATIVE
	dPrint("Final working CD: " .. iWorkingCivDiscernment)

	local sCurrentEra = Locale.ConvertTextKey(GameInfo.Eras[pPlayer:GetCurrentEra()].Description) or "Unknown Era"			
	local iIdeology, sIdeology = GetIdeology(pPlayer)
	--CloseUpPanelCivData:SetToolTipString("Report Quality: " .. GOV_INTEL_QUALITY[iWorkingCivDiscernment]);
	Controls.CloseUpPanelHeader:SetToolTipString(Locale.ConvertTextKey("TXT_KEY_GOV_OV_CUPCD_TT", GOV_INTEL_QUALITY[iWorkingCivDiscernment]))
	

	-- These elements are ALWAYS known
	Controls.CloseUpGovType:SetToolTipString("Your Government Type: " .. g_tGovernment_Types[GetGovernmentType(pActivePlayer)][1] )
	Controls.CloseUpGovType:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_GOV_TYPE", sGovernmentType) )	
		
	local iAPIdeology, sAPIdeology = GetIdeology(pActivePlayer)	
	Controls.CloseUpIdeology:SetToolTipString("Your Ideology: " .. sAPIdeology )	
	Controls.CloseUpIdeology:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_IDEOLOGY", sIdeology) )

	Controls.CloseUpEra:SetToolTipString("Your Current Era: " .. Locale.ConvertTextKey(GameInfo.Eras[pActivePlayer:GetCurrentEra()].Description) or "Unknown Era" )
	Controls.CloseUpEra:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_ERA", sCurrentEra) )

	Controls.CloseUpGameScore:SetToolTipString("Your Game Score: " .. NumWithCommas(pActivePlayer:GetScore()) )
	Controls.CloseUpGameScore:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_GAME_SCORE", pPlayer:GetScore()) )
	
	-- Populate column 1 of CUR
	Controls.CloseUpPopulation:SetToolTipString("Your Population: " .. NumWithCommas(pActivePlayer:GetRealPopulation()) )
	local sDisplayAssessment = AdjustAndFormatAssessment(pActivePlayer:GetRealPopulation(), pPlayer:GetRealPopulation(), iCivDiscernment, ipPlayerID )
	Controls.CloseUpPopulation:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_POPULATION",				sDisplayAssessment ) )
	
	Controls.CloseUpCityCount:SetToolTipString("Your City Count: " .. pActivePlayer:GetNumCities() )
	sDisplayAssessment = AdjustAndFormatAssessment(pActivePlayer:GetNumCities(), pPlayer:GetNumCities(), iCivDiscernment, ipPlayerID )	
	Controls.CloseUpCityCount:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_CITY_COUNT",				sDisplayAssessment ) )

	Controls.CloseUpLandArea:SetToolTipString("Your Land Area: " .. NumWithCommas(pActivePlayer:GetNumPlots() * 10000) )
	sDisplayAssessment = AdjustAndFormatAssessment(math.floor(pActivePlayer:GetNumPlots() * 10000), math.floor(pPlayer:GetNumPlots() * 10000), iCivDiscernment, ipPlayerID )
	Controls.CloseUpLandArea:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_LAND_AREA",					sDisplayAssessment ) )
	
	Controls.CloseUpHappiness:SetToolTipString("Your Happiness: " .. NumWithCommas(pActivePlayer:GetExcessHappiness()) )
	sDisplayAssessment = AdjustAndFormatAssessment(pActivePlayer:GetExcessHappiness(), pPlayer:GetExcessHappiness(), iCivDiscernment, ipPlayerID )
	Controls.CloseUpHappiness:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_HAPPINESS",					sDisplayAssessment ) )
	
	Controls.CloseUpTechsDiscovered:SetToolTipString("Your Discovered Tech Count: " .. Teams[pActivePlayer:GetTeam()]:GetTeamTechs():GetNumTechsKnown() )
	sDisplayAssessment = AdjustAndFormatAssessment(Teams[pActivePlayer:GetTeam()]:GetTeamTechs():GetNumTechsKnown(), Teams[pPlayer:GetTeam()]:GetTeamTechs():GetNumTechsKnown(), iCivDiscernment, ipPlayerID )
	Controls.CloseUpTechsDiscovered:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_TECHS_DISCOVERED",	sDisplayAssessment ) )
	
	Controls.CloseUpPoliciesEnacted:SetToolTipString("Your Enacted Policy Count: " .. pActivePlayer:GetNumPolicies() )
	sDisplayAssessment = AdjustAndFormatAssessment(pActivePlayer:GetNumPolicies(), pPlayer:GetNumPolicies(), iCivDiscernment, ipPlayerID )
	Controls.CloseUpPoliciesEnacted:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_POLICIES_ENACTED",	sDisplayAssessment ) )


	-- Populate column 2 of CUR	
	Controls.CloseUpCulture:SetToolTipString("Your Culture per Turn: " .. NumWithCommas(pActivePlayer:GetTotalJONSCulturePerTurn()) )
	sDisplayAssessment = AdjustAndFormatAssessment(pActivePlayer:GetTotalJONSCulturePerTurn(), pPlayer:GetTotalJONSCulturePerTurn(), iCivDiscernment, ipPlayerID )
	Controls.CloseUpCulture:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_CULTURE",						sDisplayAssessment ) )	
	
	Controls.CloseUpScience:SetToolTipString("Your Science per Turn: " .. NumWithCommas(pActivePlayer:GetScience()) )
	sDisplayAssessment = AdjustAndFormatAssessment(pActivePlayer:GetScience(), pPlayer:GetScience(), iCivDiscernment, ipPlayerID )
	Controls.CloseUpScience:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_SCIENCE",						sDisplayAssessment) )

	Controls.CloseUpGOH:SetToolTipString("Your Gold on Hand: " .. NumWithCommas(pActivePlayer:GetGold()) )
	sDisplayAssessment = AdjustAndFormatAssessment(pActivePlayer:GetGold(), pPlayer:GetGold(), iCivDiscernment, ipPlayerID )
	Controls.CloseUpGOH:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_GOH",								sDisplayAssessment ) )
	
	Controls.CloseUpGPT:SetToolTipString("Your Gold per Turn: " .. NumWithCommas(pActivePlayer:CalculateGoldRate()) )
	sDisplayAssessment = AdjustAndFormatAssessment(pActivePlayer:CalculateGoldRate(), pPlayer:CalculateGoldRate(), iCivDiscernment, ipPlayerID )
	Controls.CloseUpGPT:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_GPT",								sDisplayAssessment) )


	local tAPTradeRoutes = pActivePlayer:GetTradeRoutes()
	local tTPTradeRoutes = pPlayer:GetTradeRoutes()
	Controls.CloseUpTradeRoutes:SetToolTipString("Your Trade Route Count: " .. #tAPTradeRoutes) 
	sDisplayAssessment = AdjustAndFormatAssessment(#tAPTradeRoutes, #tTPTradeRoutes, iCivDiscernment, ipPlayerID )
	Controls.CloseUpTradeRoutes:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_TRADE_ROUTES", sDisplayAssessment ) )

	Controls.CloseUpMilitaryPop:SetToolTipString("Your Soldier Population: " .. NumWithCommas(math.floor( math.sqrt(pActivePlayer:GetMilitaryMight()) * 2000 )) )
	sDisplayAssessment = AdjustAndFormatAssessment(math.floor( math.sqrt(pActivePlayer:GetMilitaryMight()) * 2000 ), math.floor( math.sqrt(pPlayer:GetMilitaryMight()) * 2000 ), iCivDiscernment, ipPlayerID )
	Controls.CloseUpMilitaryPop:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_MILITARY_POP",			sDisplayAssessment ))
	
	Controls.CloseUpNumMilUnits:SetToolTipString("Your Military Unit Count: " .. NumWithCommas(pActivePlayer:GetNumMilitaryUnits()) )
	sDisplayAssessment = AdjustAndFormatAssessment(pActivePlayer:GetNumMilitaryUnits(), pPlayer:GetNumMilitaryUnits(), iCivDiscernment, ipPlayerID )
	Controls.CloseUpNumMilUnits:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_NUM_MIL_UNITS",			sDisplayAssessment) )
			
	Controls.CloseUpNumUnitsNoSupply:SetToolTipString("Your Unit Out of Supply Count: " .. pActivePlayer:GetNumUnitsOutOfSupply() )
	sDisplayAssessment = AdjustAndFormatAssessment(pActivePlayer:GetNumUnitsOutOfSupply(), pPlayer:GetNumUnitsOutOfSupply(), iCivDiscernment, ipPlayerID )
	Controls.CloseUpNumUnitsNoSupply:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_NUM_UNITS_NO_SUPPLY", sDisplayAssessment) )
	
	Controls.CloseUpNumDiplomats:SetToolTipString("Your Spy Count: " .. pActivePlayer:GetNumSpies() )
	sDisplayAssessment = AdjustAndFormatAssessment(pActivePlayer:GetNumSpies(), pPlayer:GetNumSpies(), iCivDiscernment, ipPlayerID )
	Controls.CloseUpNumDiplomats:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_NUM_SPIES",			sDisplayAssessment) )
		
	local iAPGovStartTurn = g_tSavedData.GetValue(string.format("GOV_OV_GOV_AGE_PLAYER_%02i", pActivePlayer:GetID() ) ) 
	if not iAPGovStartTurn then iAPGovStartTurn = Game.GetStartTurn() end						-- Get game turn old govt started, but use the game's starting turn if nothing found
	local iAPGovAge = Game.GetGameTurn() - iAPGovStartTurn	

	local iTPGovStartTurn = g_tSavedData.GetValue(string.format("GOV_OV_GOV_AGE_PLAYER_%02i", pPlayer:GetID() ) ) 
	if not iTPGovStartTurn then iTPGovStartTurn = Game.GetStartTurn() end						-- Get game turn old govt started, but use the game's starting turn if nothing found
	local iTPGovAge = Game.GetGameTurn() - iTPGovStartTurn	
	
	Controls.CloseUpNumSpies:SetToolTipString("Your Government Age: " .. iAPGovAge )
	sDisplayAssessment = AdjustAndFormatAssessment(iAPGovAge, iTPGovAge, iCivDiscernment, ipPlayerID )
	Controls.CloseUpNumSpies:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_GOV_AGE", iTPGovAge ) )

end -- =============================================================================		PopulateLowerPanelforLessKnownCiv()


function PopulateLowerPanelforWellKnownCiv(pPlayer, sGovernmentType, iCivDiscernment )		-- For Advanced (6), Fluent (7), and Native (8) Civ Discernments
	Controls.CloseUpPanelCivData:SetHide(false)
	Controls.CloseUpPanelUnknownCivData:SetHide(true)

	local pActivePlayer = Players[Game.GetActivePlayer()]									-- Get the Active Player pointer
	--local iGetCivilizationalDiscernmentAdjustment = GetCivilizationalDiscernmentAdjustment(pPlayer) or 0
	--local iWorkingCivDiscernment = iCivDiscernment + iGetCivilizationalDiscernmentAdjustment
	--dPrint("PLPWellKC. CD: " .. iCivDiscernment .. "     Adjustment: " .. iGetCivilizationalDiscernmentAdjustment .. "     Working CD: " .. iWorkingCivDiscernment)
	--if (iCivDiscernment == GOV_CD_ADVANCED) then iWorkingCivDiscernment = GOV_CD_ADVANCED end		-- Can never adjust to FLUENT or NATIVE
	
	local sCurrentEra = Locale.ConvertTextKey(GameInfo.Eras[pPlayer:GetCurrentEra()].Description) or "Unknown Era"					
	local iIdeology, sIdeology = GetIdeology(pPlayer)
	Controls.CloseUpPanelHeader:SetToolTipString(Locale.ConvertTextKey("TXT_KEY_GOV_OV_CUPCD_TT", GOV_INTEL_QUALITY[iCivDiscernment]))
	
	-- These elements are ALWAYS known
	Controls.CloseUpGovType:SetToolTipString("Your Government Type: " .. g_tGovernment_Types[GetGovernmentType(pActivePlayer)][1] )
	Controls.CloseUpGovType:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_GOV_TYPE", sGovernmentType) )
	
	local iAPIdeology, sAPIdeology = GetIdeology(pActivePlayer)	
	Controls.CloseUpIdeology:SetToolTipString("Your Ideology: " .. sAPIdeology )	
	Controls.CloseUpIdeology:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_IDEOLOGY", sIdeology) )

	Controls.CloseUpEra:SetToolTipString("Your Current Era: " .. Locale.ConvertTextKey(GameInfo.Eras[pActivePlayer:GetCurrentEra()].Description) or "Unknown Era" )
	Controls.CloseUpEra:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_ERA", sCurrentEra) )

	Controls.CloseUpGameScore:SetToolTipString("Your Game Score: " .. NumWithCommas(pActivePlayer:GetScore()) )
	Controls.CloseUpGameScore:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_GAME_SCORE", NumWithCommas(pPlayer:GetScore()) ) )
	
	local tTradeRoutes = pPlayer:GetTradeRoutes()
	local tAPTradeRoutes = pActivePlayer:GetTradeRoutes()
	Controls.CloseUpTradeRoutes:SetToolTipString("Your Trade Route Count: " .. #tAPTradeRoutes) 
	Controls.CloseUpTradeRoutes:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_TRADE_ROUTES", #tTradeRoutes ) )
	
	-- Determine the Adjustments for this game turn (amount and direction)
	local fAdjustmentDirection = GetStableRandomPercent(-1, false)							-- Get a stable random num (.0 to .9); first argument (iSeed) is -1 (the function will use the Game Turn); second argument (bIsInteger = false) to return a float
	local iAdjustmentDirection = 1															-- The adjustment defaults to a positive number (adding variance)
	if (fAdjustmentDirection <= .5) then													-- If the fAdjDirect is < .5, then...
		iAdjustmentDirection = -1															-- The adjustment will be a negative number (subtracting variance)
	end

	local fAdjustmentPercentage = GetStableRandomPercent()									-- Get a rand num, range: .0 to .9; same number for each game turn
	local fFinalAdjustmentPercentage = g_tCiv_Discernment[iCivDiscernment][3] * fAdjustmentPercentage		-- Get the Max Adjustment from table, adjusted by fAdjPercent

	-- Populate column 1 of CUR
	Controls.CloseUpPopulation:SetToolTipString("Your Population: " .. NumWithCommas(pActivePlayer:GetRealPopulation()) )
	Controls.CloseUpPopulation:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_POPULATION",				AdjustAndFormatNumber(pPlayer:GetRealPopulation(),									iAdjustmentDirection, fFinalAdjustmentPercentage) ) )
	
	Controls.CloseUpCityCount:SetToolTipString("Your City Count: " .. pActivePlayer:GetNumCities() )
	Controls.CloseUpCityCount:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_CITY_COUNT",				AdjustAndFormatNumber(pPlayer:GetNumCities(),										iAdjustmentDirection, fFinalAdjustmentPercentage) ) )
	
	Controls.CloseUpLandArea:SetToolTipString("Your Land Area: " .. NumWithCommas(pActivePlayer:GetNumPlots() * 10000) )
	Controls.CloseUpLandArea:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_LAND_AREA",					AdjustAndFormatNumber(pPlayer:GetNumPlots() * 10000,								iAdjustmentDirection, fFinalAdjustmentPercentage) ) )
	
	Controls.CloseUpHappiness:SetToolTipString("Your Happiness: " .. NumWithCommas(pActivePlayer:GetExcessHappiness()) )
	Controls.CloseUpHappiness:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_HAPPINESS",					AdjustAndFormatNumber(pPlayer:GetExcessHappiness(),									iAdjustmentDirection, fFinalAdjustmentPercentage) ) )
	
	Controls.CloseUpTechsDiscovered:SetToolTipString("Your Discovered Tech Count: " .. Teams[pActivePlayer:GetTeam()]:GetTeamTechs():GetNumTechsKnown() )
	Controls.CloseUpTechsDiscovered:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_TECHS_DISCOVERED",	AdjustAndFormatNumber(Teams[pPlayer:GetTeam()]:GetTeamTechs():GetNumTechsKnown(),	iAdjustmentDirection, fFinalAdjustmentPercentage) ) )
	
	Controls.CloseUpPoliciesEnacted:SetToolTipString("Your Enacted Policy Count: " .. pActivePlayer:GetNumPolicies() )
	Controls.CloseUpPoliciesEnacted:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_POLICIES_ENACTED",	AdjustAndFormatNumber(pPlayer:GetNumPolicies(),										iAdjustmentDirection, fFinalAdjustmentPercentage) ) )
	
	-- Populate column 2 of CUR	
	Controls.CloseUpCulture:SetToolTipString("Your Culture per Turn: " .. NumWithCommas(pActivePlayer:GetTotalJONSCulturePerTurn()) )
	Controls.CloseUpCulture:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_CULTURE",						AdjustAndFormatNumber(pPlayer:GetTotalJONSCulturePerTurn(),							iAdjustmentDirection, fFinalAdjustmentPercentage) ) )	
	
	Controls.CloseUpScience:SetToolTipString("Your Science per Turn: " .. NumWithCommas(pActivePlayer:GetScience()) )
	Controls.CloseUpScience:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_SCIENCE",						AdjustAndFormatNumber(pPlayer:GetScience(),											iAdjustmentDirection, fFinalAdjustmentPercentage) ) )
	
	Controls.CloseUpGOH:SetToolTipString("Your Gold on Hand: " .. NumWithCommas(pActivePlayer:GetGold()) )
	Controls.CloseUpGOH:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_GOH",								AdjustAndFormatNumber(pPlayer:GetGold(),											iAdjustmentDirection, fFinalAdjustmentPercentage) ) )
	
	Controls.CloseUpGPT:SetToolTipString("Your Gold per Turn: " .. NumWithCommas(pActivePlayer:CalculateGoldRate()) )
	Controls.CloseUpGPT:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_GPT",								AdjustAndFormatNumber(pPlayer:CalculateGoldRate(),									iAdjustmentDirection, fFinalAdjustmentPercentage) ) )
	
	Controls.CloseUpMilitaryPop:SetToolTipString("Your Soldier Population: " .. NumWithCommas(math.floor( math.sqrt(pActivePlayer:GetMilitaryMight()) * 2000 )) )
	Controls.CloseUpMilitaryPop:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_MILITARY_POP",			AdjustAndFormatNumber(math.floor( math.sqrt(pPlayer:GetMilitaryMight()) * 2000 ),	iAdjustmentDirection, fFinalAdjustmentPercentage) ) )
	
	Controls.CloseUpNumMilUnits:SetToolTipString("Your Military Unit Count: " .. NumWithCommas(pActivePlayer:GetNumMilitaryUnits()) )
	Controls.CloseUpNumMilUnits:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_NUM_MIL_UNITS",			AdjustAndFormatNumber(pPlayer:GetNumMilitaryUnits(),								iAdjustmentDirection, fFinalAdjustmentPercentage) ) )
	
	Controls.CloseUpNumUnitsNoSupply:SetToolTipString("Your Unit Out of Supply Count: " .. pActivePlayer:GetNumUnitsOutOfSupply() )
	Controls.CloseUpNumUnitsNoSupply:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_NUM_UNITS_NO_SUPPLY", AdjustAndFormatNumber(pPlayer:GetNumUnitsOutOfSupply(),							iAdjustmentDirection, fFinalAdjustmentPercentage) ) )
	
	Controls.CloseUpNumDiplomats:SetToolTipString("Your Spy Count: " .. pActivePlayer:GetNumSpies() )
	Controls.CloseUpNumDiplomats:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_NUM_SPIES",				AdjustAndFormatNumber(pPlayer:GetNumSpies(),										iAdjustmentDirection, fFinalAdjustmentPercentage) ) )
	
	local iAPGovStartTurn = g_tSavedData.GetValue(string.format("GOV_OV_GOV_AGE_PLAYER_%02i", pActivePlayer:GetID() ) ) 
	if not iAPGovStartTurn then iAPGovStartTurn = Game.GetStartTurn() end						-- Get game turn old govt started, but use the game's starting turn if nothing found
	local iAPGovAge = Game.GetGameTurn() - iAPGovStartTurn	

	local iTPGovStartTurn = g_tSavedData.GetValue(string.format("GOV_OV_GOV_AGE_PLAYER_%02i", pPlayer:GetID() ) ) 
	if not iTPGovStartTurn then iTPGovStartTurn = Game.GetStartTurn() end						-- Get game turn old govt started, but use the game's starting turn if nothing found
	local iTPGovAge = Game.GetGameTurn() - iTPGovStartTurn	
	
	Controls.CloseUpNumSpies:SetToolTipString("Your Government Age: " .. iAPGovAge )
	Controls.CloseUpNumSpies:LocalizeAndSetText( Locale.ConvertTextKey("TXT_KEY_GOV_OV_GOV_AGE",					AdjustAndFormatNumber(iTPGovAge,													iAdjustmentDirection, fFinalAdjustmentPercentage) ) )

end -- =============================================================================		PopulateLowerPanelforWellKnownCiv()


function PopulateLowerPanelforUnknownCiv()
	Controls.CloseUpPanelCivData:SetHide(true)
	Controls.CloseUpPanelUnknownCivData:SetHide(false)	

end -- =============================================================================		PopulateLowerPanelforUnknownCiv()


-- ========================================================================================================================
-- ========================================================================================================================
-- ========================================================================================================================
--		Shared Support Functions (in alphabetic order by function name)


function AdjustAndFormatAssessment(iActivePlayerNum, iTargetPlayerNum, iCivDiscernment, ipPlayerID )	-- Determines the display result by comparing 2 incoming numbers (active and target players); adjusts for CD value
	local sReturnAssessment = g_tCivDiscAssessment[GOV_CD_ASSESS_UNKNOWN][1]				-- Default return value to "Unknown"
	local sReturnQualifier = g_tCivDiscQualifier[GOV_CD_QUALIFIER_UNKNOWN][1]				-- Default return value to "-----"

	local iLargerValue = math.abs( math.max(iActivePlayerNum, iTargetPlayerNum) )			-- Get the larger value, make sure it's positive
	local iNumberDelta = math.abs(iActivePlayerNum - iTargetPlayerNum)						-- Get the difference between the 2 numbers
	local fDeltaPercent = iNumberDelta / (iLargerValue + 1)

	if fDeltaPercent <= .2 then																-- If numbers are < 20% apart, then
		sReturnAssessment = g_tCivDiscAssessment[GOV_CD_ASSESS_PARITY][1]					-- Assess the numbers to be on par with each other
	elseif iActivePlayerNum > iTargetPlayerNum then
		sReturnAssessment = g_tCivDiscAssessment[GOV_CD_ASSESS_INFERIOR][1]					-- Assess the Active Player to be larger
	else
		sReturnAssessment = g_tCivDiscAssessment[GOV_CD_ASSESS_SUPERIOR][1]					-- Assess the Target Player to be larger
	end

	--sReturnQualifier = g_tCivDiscQualifier[GOV_CD_QUALIFIER_ABOUT][1]						-- Assess the qualifier to be MoL
	if fDeltaPercent > .50 then																-- If numbers are < 10% apart, then
		sReturnQualifier = g_tCivDiscQualifier[GOV_CD_QUALIFIER_DRASTICALLLY][1]			-- Assess the qualifier to be Drastic
	elseif fDeltaPercent > .30 then															-- If numbers are < 10% apart, then
		sReturnQualifier = g_tCivDiscQualifier[GOV_CD_QUALIFIER_VASTLY][1]					-- Assess the qualifier to be Vast
	elseif fDeltaPercent > .15 then															-- If numbers are < 10% apart, then
		sReturnQualifier = g_tCivDiscQualifier[GOV_CD_QUALIFIER_ROUGHLY][1]					-- Assess the qualifier to be Rough
	end
	
	--dPrint("AAFA. AP Num: " .. iActivePlayerNum .. "     TP Num: " .. iTargetPlayerNum .. "     Actual Assessment: " .. sReturnAssessment .. "     Actual Qualifier: " .. sReturnQualifier )

	-- At this point, we have "correct" Qualifier/Assessment - Now - make adjustments to add some ambiguity - more with lower CDs, less with larger
		
	local fAssessmentShowChance = GetStableRandomPercent(ipPlayerID * iActivePlayerNum  , false)			-- Use pPlayer's ID as the seed; return float
	local fQualifierShowChance = GetStableRandomPercent(iTargetPlayerNum, false)			-- Use TPNum as the seed; return float	

	if iCivDiscernment == GOV_CD_INTRODUCTORY then											-- This Civ Disc (3) means there are no qualifiers, and only some assessments (25% chance)		
		if (fAssessmentShowChance <= .75) then												-- If Assessment chance is less than 75%, then...  
			sReturnAssessment = g_tCivDiscAssessment[GOV_CD_ASSESS_UNKNOWN][1]				-- Set Assessment to "Unk"
		end
		sReturnQualifier = g_tCivDiscQualifier[GOV_CD_QUALIFIER_UNKNOWN][1]					-- Automatically set to ""; the qualifier is automatically blanked out (aka "Unk")

	elseif iCivDiscernment == GOV_CD_BASIC then												-- This Civ Disc (4) means there are some qualifiers, and assessments on most entries
		if (fAssessmentShowChance <= .50) then												-- If Assessment chance is less thant 50%, then...  
			sReturnAssessment = g_tCivDiscAssessment[GOV_CD_ASSESS_UNKNOWN][1]				-- Set to "-----", or aka "Unk"
		end
		if (fQualifierShowChance <= .50) or													-- If Qualifier chance is less than 50%, OR...  
				(sReturnAssessment == g_tCivDiscAssessment[GOV_CD_ASSESS_UNKNOWN][1]) then	-- If the Assessment is "Unk", then... 
				
			sReturnQualifier = g_tCivDiscQualifier[GOV_CD_QUALIFIER_UNKNOWN][1]				-- Set to ""; the qualifier is automatically blanked out (aka "Unk")
		end		

	elseif iCivDiscernment == GOV_CD_FUNCTIONAL then										-- This Civ Disc (5) means there are mostly qualifiers, but assessments on all entries	
		if (fAssessmentShowChance <= .25) then												-- If Assessment chance is less than 25%, then...  
			sReturnAssessment = g_tCivDiscAssessment[GOV_CD_ASSESS_UNKNOWN][1]				-- Set to "-----", or aka "Unk" 
		end
		if (fQualifierShowChance <= .50) or													-- If Qualifier chance is less than 50%, OR...  (50:50 chance)
				(sReturnAssessment == g_tCivDiscAssessment[GOV_CD_ASSESS_UNKNOWN][1]) then	-- If the assessment is "Unk", then... 
				
			sReturnQualifier = g_tCivDiscQualifier[GOV_CD_QUALIFIER_UNKNOWN][1]				-- Set to ""; the qualifier is automatically blanked out (aka "Unk")
		end		
	end	
	--dPrint("AAFA. Final Assessment: " .. sReturnAssessment .. "     Final Qualifier: " .. sReturnQualifier )
	return sReturnQualifier .. sReturnAssessment
end -- =============================================================================		AdjustAndFormatAssessment()


function AdjustAndFormatNumber(iInitialNumber, iAdjustmentDirection, fFinalAdjustmentPercentage)
	local sAdjustedNumber = "Unknown"
	local iAdjustAmount = math.floor( iInitialNumber * fFinalAdjustmentPercentage  ) 
	local iTempNumber = math.floor( iInitialNumber + (iAdjustAmount * iAdjustmentDirection) )
	sAdjustedNumber = NumWithCommas(iTempNumber)											-- Comma delimit number, convert it to a string
	--dPrint("AAFN. Initial Num: " .. iInitialNumber .. "     Adj Percent: " .. fFinalAdjustmentPercentage .. "     Direction: " .. iAdjustmentDirection .. "     Adj. Amount: " .. iAdjustAmount .. "     Final Number: " .. sAdjustedNumber)
	return sAdjustedNumber
end	-- =============================================================================		AdjustAndFormatNumber()																			


function BuildStatsTable()																	-- Builds a global table of various stats used throughout the mod
	for iDX = 0, GameDefines.MAX_MAJOR_CIVS - 1, 1 do
		local pPlayer = Players[iDX]
		local iPlayerID = pPlayer:GetID()		
		g_tStatsTable[iPlayerID+1] = {}				
		if ( pPlayer ~= nil and pPlayer:IsAlive() ) then							
			g_tStatsTable[iPlayerID + 1][STAT_SCORE] = pPlayer:GetScore()
			g_tStatsTable[iPlayerID + 1][STAT_POPULATION] =	pPlayer:GetRealPopulation()
			g_tStatsTable[iPlayerID + 1][STAT_MIL_POWER] = pPlayer:GetMilitaryMight()
			g_tStatsTable[iPlayerID + 1][STAT_LITERACY] = pPlayer:GetScoreFromTechs()
		else
			g_tStatsTable[iPlayerID + 1][STAT_SCORE] = -1
			g_tStatsTable[iPlayerID + 1][STAT_POPULATION] =	-1
			g_tStatsTable[iPlayerID + 1][STAT_MIL_POWER] = -1
			g_tStatsTable[iPlayerID + 1][STAT_LITERACY] = -1
		end		
	end
	
	--for iDX = 0, GameDefines.MAX_MAJOR_CIVS - 1, 1 do
	--	local pPlayer = Players[iDX]
	--	print("iDX: " .. iDX .. "     Player: " .. pPlayer:GetCivilizationShortDescription() .. "     Score: " .. g_tStatsTable[iDX+1][STAT_SCORE] .. "    Pop: " .. g_tStatsTable[iDX+1][STAT_POPULATION] .. "     MilPow: " .. g_tStatsTable[iDX+1][STAT_MIL_POWER] .. "     TechScore: " .. g_tStatsTable[iDX+1][STAT_LITERACY])
	--end
end -- =============================================================================		BuildStatsTable()


function CanPlayerBuildGovernment(pPlayer, iGovernmentType, iTestGovType)					-- Checks if pPlayer has the prereq tech and social policy branch enabled
	local bIsPropsedGovernmentValid = false
	local bCanBuild = false	
	local bHasPolicy = false
	
	local iGovBuildingType = GameInfoTypes[ g_tGovernment_Types[iTestGovType][3] ]			
	bCanBuild = pPlayer:CanConstruct(iGovBuildingType, 0, 0, 1)								-- Can the capital legally build the gov building (i.e. player has req. tech and social policy brach enabled)?

	local iRequiredSocialPolicyType = GameInfoTypes[ g_tGovernment_Types[iTestGovType][7] ] or -1	-- Get the additional required policy, if there is one
	if iRequiredSocialPolicyType == -1 then													-- Does this government require and additional Social Policy? Val of -1 means it doesn't
		bHasPolicy = true																	-- Set to true if no additional Social Policy is required
	else
		bHasPolicy = pPlayer:HasPolicy(iRequiredSocialPolicyType)							-- If the gov requires an additional SP, then does pPlayer have it?
	end

	if bCanBuild and bHasPolicy then 
		bIsPropsedGovernmentValid = true
	end

	if (iGovernmentType == iTestGovType) or (iTestGovType == GOV_ANARCHY) then				-- If (the proposed gov is the current gov) or (proposed gov is Anarchy [which is always available], then...
		bIsPropsedGovernmentValid = true
	end

	return bIsPropsedGovernmentValid
end	-- =============================================================================		CanPlayerBuildGovernment()


function ClearGovernmentBuildings(pPlayer)													-- Removes all gov buildings from pPlayer's capital
	print("Gov Change.  Removing buildings...")
	local pCapitalCity = pPlayer:GetCapitalCity()
		
	if pCapitalCity then
		for iDX = GOV_DESPOTISM, GOV_TECHNOCRACY, 1 do										-- Loop through all govt building, skip Anarchy (don't want to remove the Palace)		
			local iGovBuildingType = GameInfoTypes[ g_tGovernment_Types[iDX][3] ]
			pCapitalCity:SetNumRealBuilding(iGovBuildingType, 0)							-- Removes building type from city	
			--print("iDX: " .. iDX .. "     Gov Building Type: " .. GameInfoTypes[ g_tGovernment_Types[iDX][3] ])
		end
	else																					-- No capital city for pPlayer, so return early
		dPrint("ClearGovernmentBuildings. ERROR! No Capital for " .. pPlayer:GetCivilizationShortDescription())
		return	
	end
end -- =============================================================================		ClearGovernmentBuildings()


function GetCivilizationalDiscernment(pTargetPlayer)										-- Determines pPlayers Civ Discernment value for Active player
	local iReturnCD = GOV_CD_UNKNOWN														-- Default return is "Unknown"	
	local pActivePlayer = Players[Game.GetActivePlayer()]									-- Get the Active Player pointer
	if pActivePlayer then																	-- If AP pointer is valid, then...		
		local iActivePlayerType = GameInfoTypes[ pActivePlayer:GetCurrentEra() ] or 0		-- Get the Era of the Active Player (human), or "Ancient" Era on error
			
		--	Test #7:  Is the CD value = GOV_CD_NATIVE?  Only if the Lower Panel is displaying the human player of a team member of the human player
		if (pActivePlayer:GetID() == pTargetPlayer:GetID() ) or								-- If the AP is the TP  OR
			( pActivePlayer:GetTeam() == pTargetPlayer:GetTeam() ) then						--		If the AP is on the same team as TP, then...
			--dPrint("Active Civ: " .. pActivePlayer:GetCivilizationShortDescription() .. "     Target Civ: " .. pTargetPlayer:GetCivilizationShortDescription() .. "     Civ Discernment: " .. g_tCiv_Discernment[GOV_CD_NATIVE][1])
			return GOV_CD_NATIVE
		end

		--	Subsequent tests must have a target player capital, otherwise the default return (unk) goes back (for 'complete kills' option)
		local pTargetCapital = pTargetPlayer:GetCapitalCity()									-- Get pPlayer's capital
		if pTargetCapital then
			local iCapitalX = pTargetCapital:GetX()
			local iCapitalY = pTargetCapital:GetY()
			
			local tSpyInfo = pActivePlayer:GetEspionageSpies()								-- Get Spy Table data for pPlayer		
			local iSpyCount = #tSpyInfo or 0												-- Count rows in tSpyInfo for spy count	
			local bDiplomatInCapital = false
			local bSpyInCapital = false
			local bDiplomatInCiv = false
			local bSpyInCiv = false
			local bHasEmbassy = false

			if iSpyCount > 0 then															-- If there's 1 or more spies, then...
				for i,v in ipairs(tSpyInfo) do													-- Cycle through all spies in the table							
					-- Test for Diplomat or Spy.  Get the "spy city" - then test if Target Player owns it, then if so - is the agent a Diplomat or Spy?				
					local pSpyCityPlot = Map.GetPlot(v.CityX, v.CityY)						-- Get the Plot for the spy's CityX and CityY values
					if pSpyCityPlot then													-- If the Plot is valid, then...  
						local pSpyCity = pSpyCityPlot:GetPlotCity()							-- Get the City pointer at that Plot
						if pSpyCity then													-- If the City pointer is valid, then...

							--dPrint("i: " .. i .. "    City name: " .. pSpyCity:GetName() .. "     Spy Name: " .. v.Name)

							if pSpyCity:GetOwner() == pTargetPlayer:GetID() then			-- If the spy city owner's ID matches the Target Player ID, then...
							
								--  Next serveral lines are an extensive test required to ensure the Diplomat/Spy is "established" - spies in transit, etc., cannot report back
								local sSpyMission = Locale.ConvertTextKey(v.State) or "Unknown"				-- Get spy's mission
								local sAgentActivity = ""
								local iSpyTurnsLeft = v.TurnsLeft or 0

								if (sSpyMission == Locale.Lookup("TXT_KEY_SPY_STATE_SURVEILLANCE") and iSpyTurnsLeft < 0) then
									sAgentActivity = Locale.Lookup("TXT_KEY_SPY_STATE_ESTABLISHED_SURVEILLANCE")
								else
									sAgentActivity = Locale.Lookup(sSpyMission)	
								end		
	
								if (sAgentActivity == Locale.Lookup("TXT_KEY_SPY_STATE_GATHERING_INTEL")) or									-- Spy is gathering intel
										(sAgentActivity == Locale.Lookup("TXT_KEY_SPY_STATE_RIGGING_ELECTION")) or							-- Spy is rigging election
										(sAgentActivity == Locale.Lookup("TXT_KEY_SPY_STATE_SCHMOOZING")) or								-- Diplomat is Schmoozing 				
										(sAgentActivity == Locale.Lookup("TXT_KEY_SPY_STATE_ESTABLISHED_SURVEILLANCE")) or					-- Spy has established surveillance
										(sAgentActivity == Locale.Lookup("TXT_KEY_SPY_STATE_ESTABLISHED_SURVEILLANCE_PROGRESS_BAR")) then	-- Spy has no techs to steal
			
									if v.IsDiplomat then										-- If this spy is a Diplomat, then...
										if pSpyCity:IsCapital() then							-- If the spy City is the capital, then
											bDiplomatInCapital = true							-- Set the DiploInCap to true
											break												-- Exit the spy loop early								
										end								
									else														-- If the spy is NOT a Diplomat, then...
										if pSpyCity:IsCapital() then							-- If the spy City is the capital, then
											bSpyInCapital = true								-- Set the SpyInCap to true
											break												-- Exit the spy loop early
										else													-- If the spy City is NOT the capital, then
											bSpyInCiv = true									-- Set the SpyInCic to true
											break												-- Exit the spy loop early
										end								
									end
								end							
							end
						end
					end
				end
			end			
			
			if (Teams[pActivePlayer:GetTeam()]:HasEmbassyAtTeam(pTargetPlayer:GetTeam())) then	-- Does the AP have an embassy with the TP?
				bHasEmbassy = true
			end
			
			if pActivePlayer:GetTeam() == pTargetPlayer:GetTeam() then						-- If AP and TP are on the same team, then...
				iReturnCD = GOV_CD_NATIVE													-- Assign a "Native" Civ Disc., i.e. they have completely accurate data on each other
			end
			
			if bDiplomatInCapital then														-- Test #7:  Is the Active Player GOV_CD_FLUENT with pPlayer?  Must have a Diplomat in the capital
				iReturnCD = GOV_CD_FLUENT				
			
			elseif bSpyInCapital then														-- Test #6:  Is the Active Player GOV_CD_ADVANCED with pPlayer?  Must have a Spy in the capital
				iReturnCD = GOV_CD_ADVANCED

			elseif bSpyInCiv then															-- Test #5:  Is the Active Player GOV_CD_FUNCTIONAL with pPlayer?  Must have a Spy in any non-capital city
				iReturnCD = GOV_CD_FUNCTIONAL

			elseif bHasEmbassy then															-- Test #4:  Is the Active Player GOV_CD_BASIC with pPlayer?  Must have an established embassy with target civ
				iReturnCD = GOV_CD_BASIC
	
			elseif Teams[pTargetPlayer:GetTeam()]:IsHasMet(pActivePlayer:GetTeam())	 then	-- Test #3:  Is the Active Player GOV_CD_INTRODUCTORY with pPlayer?  Must have met the target civ
				iReturnCD = GOV_CD_INTRODUCTORY
										
			else																			-- Test #2:  Active Player GOV_CD_UNMET with pPlayer! No other test worked, but the target player still has a capital
				iReturnCD = GOV_CD_UNMET
			end
		end
	end
	--dPrint("Active Civ: " .. pActivePlayer:GetCivilizationShortDescription() .. "     Target Civ: " .. pTargetPlayer:GetCivilizationShortDescription() .. "     Civ Discernment: " .. g_tCiv_Discernment[iReturnCD][1])
	return iReturnCD																		-- Will return GOV_CD_UNKNOWN if no other test changes the value
end -- =============================================================================		GetCivilizationalDiscernment()


function GetCivilizationalDiscernmentAdjustment(pTargetPlayer)								-- Adjust the overall CD based on Trade Routes and Eras (later Eras get better info)
	local iCivDiscernmentAdjustment = 0														-- Return default is 0 (no adjustment)
	local pActivePlayer = Players[Game.GetActivePlayer()]									-- Get the Active Player pointer

	local fTradePartnerScore = TradePartnerScore (pTargetPlayer)
	iCivDiscernmentAdjustment = iCivDiscernmentAdjustment + fTradePartnerScore

	local iActivePlayerEra = pActivePlayer:GetCurrentEra()									-- Add to CD, based on Active Player's current Era
	if iActivePlayerEra == GameInfoTypes.ERA_INDUSTRIAL then								-- Use "brute force" test, in case other mods adjust Eras
		iCivDiscernmentAdjustment = iCivDiscernmentAdjustment + .5
	elseif iActivePlayerEra == GameInfoTypes.ERA_MODERN then
		iCivDiscernmentAdjustment = iCivDiscernmentAdjustment + 1
	elseif iActivePlayerEra == GameInfoTypes.ERA_POSTMODERN then
		iCivDiscernmentAdjustment = iCivDiscernmentAdjustment + 1.5
	elseif iActivePlayerEra == GameInfoTypes.ERA_FUTURE then
		iCivDiscernmentAdjustment = iCivDiscernmentAdjustment + 2
	end
	
	if Teams[pActivePlayer:GetTeam()]:IsAtWar(pTargetPlayer:GetTeam()) then					-- If the AP is at war with the TP, then...
		iCivDiscernmentAdjustment = iCivDiscernmentAdjustment - 1							-- Decrement the adjustment by 1
	end
	
	if (pActivePlayer:IsDoF(pTargetPlayer:GetID())) then									-- Is there a DoF between the AP and TP? Then...
	iCivDiscernmentAdjustment = iCivDiscernmentAdjustment + 1								-- Increment the adjustment by 1
	end
		
	dPrint("GetCivDiscAdj. Adjusted CD: " .. iCivDiscernmentAdjustment .. "     Floored: " .. math.floor(iCivDiscernmentAdjustment + .5))
	iCivDiscernmentAdjustment = math.floor(iCivDiscernmentAdjustment + .5)					-- Round up the Adjustment to the nearest integer

	return iCivDiscernmentAdjustment

end -- =============================================================================		GetCivilizationalDiscernmentAdjustment()


function GetGovernmentChangeCost(pPlayer)													-- Returns the cost in Gold to change govenment (represents lost productivity)
	local iCurrentEraPlusOne = pPlayer:GetCurrentEra() + 1
	local iGoldCost = math.floor( (( iCurrentEraPlusOne * (Game.GetHandicapType()+1) * (Game.GetGameSpeedType()+1) ) * g_iCostCoefficient)  *  iCurrentEraPlusOne  )
	if iGoldCost < 30 then iGoldCost = 30 end												-- Put a lower bound, in case a new Era is introduced
	if iGoldCost > 5000 then iGoldCost = 5000 end											-- Put an upper bound, in case a new Era is introduced
	return iGoldCost
end -- =============================================================================		GetGovernmentChangeCost()


function GetGovernmentType(pPlayer)															-- Return the Gov index (2 .. 12) or GOV_ANARCHY (1) if not found; will never be GOV_UNKNOWN
	local iGovType = GOV_ANARCHY
	local pCapital = pPlayer:GetCapitalCity()
	if pCapital then
		--print("Cap City: " .. pCapital:GetName() )
		for iDX = GOV_DESPOTISM, GOV_TECHNOCRACY, 1 do		
			local iGovBuildingType = GameInfoTypes[ g_tGovernment_Types[iDX][3] ]
			--print("iDX: " .. iDX .. "     Type?: " .. GameInfoTypes[ g_tGovernment_Types[iDX][3] ])

			if pPlayer:CountNumBuildings(iGovBuildingType) > 0 then
				--local buildingDesc = GameInfo.Buildings[iGovBuildingTtype].Description
				--print("GetGovType.  Civ: " .. pPlayer:GetCivilizationShortDescription() .. "     Gov Type: " .. g_tGovernment_Types[iDX][1] .. "     Bldg. Desc: " .. buildingDesc)
				iGovType = iDX
				break
			end
		end
	end
	--print("Returning Gov Type: " .. g_tGovernment_Types[iGovType][1] )
	return iGovType
end -- =============================================================================		GetGovernmentType()


function GetIdeology(pPlayer)																-- Returns the Ideology ID and Description.
	local sIdeology = "None"																-- Default Desc is "None"
	local iIdeology = pPlayer:GetLateGamePolicyTree() or -1									-- Default ID is -1 from GLGPT method, but just in case...
	if (iIdeology ~= PolicyBranchTypes.NO_POLICY_BRANCH_TYPE) then							-- If there is a Desc available, then...		
		sIdeology = Locale.Lookup(GameInfo.PolicyBranchTypes[iIdeology].Description)		-- Set the return value to the appropriate Ideology Desc
	end
	return iIdeology, sIdeology

end -- =============================================================================	GetIdeology(pPlayer)


function NumWithCommas(i)																	-- Returns a string with commas every 3 numbers for input argument 'i' 
	return tostring(i):reverse():gsub("%d%d%d", "%1,"):reverse():gsub("^,", "")
end -- =============================================================================		NumWithCommas()


function GetOrdinalNumber (iNumber)															-- Returns the English ordinal for the incoming number
	local sSuffix = "th"																	-- Default suffix for ordinal numbers	
	local sTempNumber = tostring(iNumber)
	local iLastDigit = tonumber(string.sub(sTempNumber, -1))
	
	if (iLastDigit == 1) and (iNumber ~= 11) then																	-- Change the iNumber suffix if needed
		sSuffix = "st"
	elseif (iLastDigit == 2) and (iNumber ~= 12) then
		sSuffix = "nd"
	elseif (iLastDigit == 3) and (iNumber ~= 13) then
		sSuffix = "rd"
	end	
		
	return sTempNumber .. sSuffix
end -- =============================================================================		GetOrdinalNumber()

function GetRank( iStat, iPlayer )															-- Rerturn's iPlayer's rank for iStat; assumes a -1 value for iPlayers not to be considered!
	local iPlayerVal = g_tStatsTable[iPlayer+1][iStat]
    local iRank = 1
	local iActivePlayers = 0
    for i = 0, GameDefines.MAX_MAJOR_CIVS - 1 do        
        if( g_tStatsTable[i+1][iStat] > iPlayerVal ) then
            iRank = iRank + 1
        end
		if g_tStatsTable[i+1][iStat] > -1 then
			iActivePlayers = iActivePlayers + 1
		end			
    end    
    return iRank, iActivePlayers
end -- =============================================================================		GetRank()


function GetStableRandomPercent(iSeed, bIsInteger)											-- Returns a "random" number, based on the iSeed; return val range = .0 to .9, or 0 to 9 if bIsInteger is true
																							-- If the first argument (iSeed) is < 1 (the function will use the Game Turn as the seed); second argument (bIsInteger = false) to return a float
	fStableRandomNum = .1																	-- Intent is to make sure the number doesn't change on the same game turn, in case a player enters the GO UI multiple times on the same turn	
	if ( (not iSeed) or (iSeed < 1) ) then													-- If iSeed is nil or less than 1 then...
		iSeed = Game.GetGameTurn() or 123													-- Set iSeed to the game turn or 123 if that fails for some crazy reason
	end	
	local iWorkingSeed = math.abs(iSeed) or 123												-- Ensure the seed is a positive number (just in case), or 123 if there's some kind of problem
	local fGTSquareRoot = math.sqrt(iWorkingSeed + 107)										-- Get square root of iSeed
	local sTempAdjPercent = tostring(fGTSquareRoot)											-- Convert the number to a string
	local iLastDigit = tonumber(string.sub(sTempAdjPercent, -1))							-- Grab the last digit of the sqrt
	local fStableRandomNum = iLastDigit / 10												-- Float version of last (digit / 10)

	if fStableRandomNum > .9 then fStableRandomNum = .9 end									-- Additional checks to ensure the return value is in the proper range
	if fStableRandomNum < 0 then fStableRandomNum = 0 end	

	if bIsInteger then fStableRandomNum = math.root(fStableRandomNum * 10) end				-- If bIsInteger, scale up the result (should be 0 .. 9)

	--dPrint("GSRP. WorkSeed: " .. iWorkingSeed .. "    Temp Percent: " .. sTempAdjPercent .. " Last Digit: " .. iLastDigit .. "     Rand Num: " .. fStableRandomNum)

	return fStableRandomNum 

end	-- =============================================================================		GetStableRandomPercent() 


function ImposeAnarchy(pPlayer)																-- Allegedly imposes anarchy on pPlayer for a "few" turns; limited production/gold/science
	local iAnarchyDuration = math.floor ( (pPlayer:GetCurrentEra() /2 ) + (Game.GetHandicapType() / 2) ) - math.random(0, 2)	-- Duration depends on Era & Difficulty, minus a little randomness
	
	if iAnarchyDuration > 0 then
		if pPlayer:IsAnarchy() then																-- If civ is already in anarchy, this will add anarchyDuration turns
			pPlayer:ChangeAnarchyNumTurns(iAnarchyDuration)
		else																					-- If not already in anarchy, will set anarchy on for anarchyDuration turns
			pPlayer:SetAnarchyNumTurns(iAnarchyDuration)
		end		
	end
	return iAnarchyDuration
end	-- =============================================================================		ImposeAnarchy() 


function IsValidGovernment(sIdeology, iGovType)												-- Determines if the proposed gov is "compatible" with the AP's ideology
	local bIsValidGov = false
	local sAllowedIdeologies = g_tGovernment_Types[iGovType][5]								-- [5] is the allowed code = "AFNO" for all ideologies valid
	local iIdCharPos = string.find( sAllowedIdeologies, string.sub(sIdeology, 1, 1) ) or -1	-- Get the first letter of the AP's Ideology - then look for it in the valid code; return -1 if not found; if positive, the ideology is in the valid code
	if (iIdCharPos > 0) then																-- If positive, the ideology is in the valid code, the gov is valide
		bIsValidGov = true
	end
	--dPrint("IsValidGov. Ideology: " .. sIdeology .. "     Gov Type: " .. g_tGovernment_Types[iGovType][1] .. "     Allowed Ideologies: " .. sAllowedIdeologies .. "     CharPos: " .. iIdCharPos)
	return bIsValidGov
end -- =============================================================================		IsValidGovernment()


function PenalizGovernmentForChanging(pPlayer, iGoldCost)								-- Reduces stats when gov changes; returns number of turns in Anarchy
	local iCurrentGold = pPlayer:GetGold()												-- Decrements change cost in gold; zeroes out culture, faith, GA progress, and research progress
	local iNewGold =  iCurrentGold - iGoldCost											-- Decrement gold, even if there isn't enough
	if iNewGold < 0 then iNewGold = 0 end												-- Ensure bottoms out at 0
	pPlayer:SetGold(iNewGold)															-- Set the new value	
	pPlayer:SetJONSCulture(0)
	pPlayer:SetFaith(0)
	pPlayer:SetGoldenAgeProgressMeter(1)	
	local pPlayerTeam = Teams[pPlayer:GetTeam()]
	local pTeamTechs = pPlayerTeam:GetTeamTechs();
	local iCurrentTech = pPlayer:GetCurrentResearch();
	pTeamTechs:SetResearchProgress(iCurrentTech, 0);
	
	iAnarchyDuration = ImposeAnarchy(pPlayer)
	return iAnarchyDuration
end -- =============================================================================		PenalizGovernmentForChanging()


function RemovePromotionFromOldGovernment(pPlayer, iSelectedGovernmentID)					-- Manages unit promotions for different governments
	local iGovPromotionID = GameInfoTypes[ g_tGovernment_Types[iSelectedGovernmentID][6] ]	-- The 6th element is the Promotion keys
	--dPrint("Removing Promotions. Player: " .. pPlayer:GetCivilizationShortDescription() .. "     New Government: " .. g_tGovernment_Types[iSelectedGovernmentID][1] .. "     Promotion: " .. GameInfo.UnitPromotions[iGovPromotionID].Description .. "    ID: " .. iGovPromotionID)
	for pUnit in pPlayer:Units() do															-- Loop through all pPlayer's units
		for iDX = GOV_DESPOTISM, GOV_TECHNOCRACY, 1 do										-- Loop through all Government types.  Very brute force, but guarantees no erroneous promotions			
			if iDX ~= iSelectedGovernmentID then											-- If loop index ~= new gov, then...			
				pUnit:SetHasPromotion(GameInfoTypes[ g_tGovernment_Types[iDX][6] ], false)	-- Remove this promotion from the unit; [6] is the Promotion ID		
			end			
		end
	end	
end -- =============================================================================		RemovePromotionFromOldGovernment()


function SentNotification(iKey, NotificationStringLong, NotificationStringFading, iX, iY )	-- Displays notification for the right side of the screen, iKey parameter determines which notification displays	
	if iKey == 1 then																		-- Display blue exclamation pint
		Players[Game.GetActivePlayer()]:AddNotification(NotificationTypes.NOTIFICATION_GENERIC , NotificationStringLong, NotificationStringFading, iX, iY)
	elseif iKey == 2 then																	-- Display blue capitol building w/ yellow flag raising
		Players[Game.GetActivePlayer()]:AddNotification(NotificationTypes.NOTIFICATION_CAPITAL_RECOVERED , NotificationStringLong, NotificationStringFading, iX, iY)
	end
end -- =============================================================================		SentNotification()


function TradePartnerScore (pPlayer)														-- Determines how much trade is occuring between AP and TP; impacts Civ Disc.
	local pActivePlayer = Players[Game.GetActivePlayer()]									-- Get the Active Player pointer
	local iTargetPlayerID = pPlayer:GetID()
	local fTotalRoutesTo = 0
	local fTotalRoutesFrom = 0

	local tToTRdata = pActivePlayer:GetTradeRoutes()
	for i,v in ipairs(tToTRdata) do
		if v.ToCity:GetOwner() == iTargetPlayerID then
			if v.ToCity:IsCapital() then		
				fTotalRoutesTo = fTotalRoutesTo + .50
			else
				fTotalRoutesTo = fTotalRoutesTo + .25
			end
		end
	end

	local tFromTRdata = pActivePlayer:GetTradeRoutesToYou();
	for i,v in ipairs(tFromTRdata) do
		if v.FromCity:GetOwner() == iTargetPlayerID then
			if v.FromCity:IsCapital() then
				fTotalRoutesFrom = fTotalRoutesFrom + .25
			else
				fTotalRoutesFrom = fTotalRoutesFrom + .125
			end
		end
	end

	--dPrint("Trade Partner Score. AP: " .. pActivePlayer:GetCivilizationShortDescription() .. "     Tgt Player: " .. pPlayer:GetCivilizationShortDescription() .. "     TR To: " .. fTotalRoutesTo .. "     TR From: " .. fTotalRoutesFrom )
	return (fTotalRoutesTo + fTotalRoutesFrom)

end -- =============================================================================		TradePartnerScore()


