-- GovernmentSelectPopup.lua
-- Author: sman
-- DateCreated: 08/01/18 6:02:11 AM
--------------------------------------------------------------

-- ====================================================================================================================================
--		Select Government popup - returns a building ID or -1 (random building)

--local g_SortTable																			-- Global table used in sort function
local g_ActivePlayer																		-- Global AP pointer
local g_StartingGovernmentID = 1															-- Global Gov ID of AP when the popup first launched
local g_SelectedGovernmentID = 1															-- Global Gov ID of type currently in the shown icon, before pressing the "Select" button


-- ====================================================================================================================================
--		Input Handlers

function OnGovOVSelGovOK()																	-- "Confirm" button pressed, building selection is final
	Controls.GovOVSelGovPopup:SetHide(true)													-- Close the popup
	ImplementGovernmentChange(g_ActivePlayer, g_StartingGovernmentID, g_SelectedGovernmentID)	-- Process the change in government
end
Controls.GovOVSelGovOK:RegisterCallback(Mouse.eLClick, OnGovOVSelGovOK)						-- Register the callback


function OnGovOVSelGovCancel()																-- "Confirm" button pressed, building selection is final
	Controls.GovOVSelGovPopup:SetHide(true)													-- Close the popup
end
Controls.GovOVSelGovCancel:RegisterCallback(Mouse.eLClick, OnGovOVSelGovCancel)				-- Register the callback


-- ====================================================================================================================================
--		Gov Select Popup Support Functions

function SortByName(a, b)																	-- QUAD sort function - currently not used; gov's appear in "chronological" order
	local sNameA = g_SortTable[tostring(a)].Name
	local sNameB = g_SortTable[tostring(b)].Name
	return sNameA < sNameB
end


function OnGovernmentSelected(iGovernmentID)													-- "Building has been selected" callback - "selected" but not confirmed!
	Controls.GovOVSelGovMenu:GetButton():SetText(g_tGovernment_Types[iGovernmentID][1])									-- Set the "button" to the building name so it displays properly
	local iPortraitIDX = iGovernmentID  
	IconHookup( iPortraitIDX, 128, "SMAN_NATIONAL_GOVERNMENTS_ATLAS", Controls.SelectGovernmentCenterIcon )				-- Display the building/question mark in the center of the popup			
	Controls.SelectGovernmentCenterIcon:SetToolTipString( Locale.ConvertTextKey(g_tGovernment_Types[iGovernmentID][4]) )
	g_SelectedGovernmentID = iGovernmentID														-- Set the PROPOSED return value to the currently selected building	
end

		
function UpdateGovernmentList(iGovernmentType, iIdeology, sIdeology)								-- Construct a list of non-wonder buildings in pTargetCity
	--g_SortTable = {}
	Controls.GovOVSelGovMenu:ClearEntries()														-- Clear any previous values that might have been there
	local sGovOVSelGovMessage = Locale.ConvertTextKey("TXT_KEY_GOV_OV_SEL_GOV_MESSAGE", g_ActivePlayer:GetCivilizationShortDescription())	-- Update the "Message" on the popup (e.g. "Attack in Omaha")
	Controls.GovOVSelGovMessage:LocalizeAndSetText(sGovOVSelGovMessage)										-- Set the title (inside top green bar) for the popup
	IconHookup(iGovernmentType, 128, "SMAN_NATIONAL_GOVERNMENTS_ATLAS", Controls.SelectGovernmentCenterIcon)	-- Initial center icon will be "Unk"
	Controls.SelectGovernmentCenterIcon:SetToolTipString( Locale.ConvertTextKey(g_tGovernment_Types[iGovernmentType][4]) )

	for iDX = GOV_ANARCHY, GOV_TECHNOCRACY, 1 do											-- Scroll through all Governments
		if IsValidGovernment(sIdeology, iDX) and CanPlayerBuildGovernment(g_ActivePlayer, iGovernmentType, iDX) then	-- IsValidGov checks against Ideology; CanBuild checks against prereq tech and social policy branch			
			local sGovernmentName = g_tGovernment_Types[iDX][1]							-- Get the name of this Gov
			local entry = {}															-- Create an entry in the Pulldown Stacl
			Controls.GovOVSelGovMenu:BuildEntry("InstanceOne", entry)						
			--g_SortTable[tostring(entry.Button)] = {Name=sGovernmentName}					-- Add this building name to the SortTable in the right location
			entry.Button:SetVoid1(iDX)												-- Void1 on this entry
			entry.Button:SetText(sGovernmentName)											-- Set the text value for the menu item
			
		end
	end	
	
	--Controls.GovOVSelGovMenuStack:SortChildren(SortByName)											-- Sort items in the menu stack	
	Controls.GovOVSelGovMenu:GetButton():LocalizeAndSetText("TXT_KEY_CO_COSB_MENU_CHOOSE")			-- Prompt string for the pulldown menu
	Controls.GovOVSelGovMenu:CalculateInternals()													-- Popup bookkeeping	
	Controls.GovOVSelGovMenu:RegisterSelectionCallback(OnGovernmentSelected)						-- Register the OnGovernmentSelected callback
end


function ImplementGovernmentChange(pPlayer, iStartingGovernmentID, iSelectedGovernmentID)	-- This function is called after the popup is dismissed.
	dPrint("Inside IGCP.  Civ: " .. pPlayer:GetCivilizationShortDescription() .. "Original Government: " .. g_tGovernment_Types[iStartingGovernmentID][1] .. "     New government: " .. g_tGovernment_Types[iSelectedGovernmentID][1])

	if ( iStartingGovernmentID == iSelectedGovernmentID ) then								-- If not change in government, then...
		return																				-- Exit early, nothing to do
	end
	
	-- Process the change of government
	local pCapitalCity = pPlayer:GetCapitalCity()
	ClearGovernmentBuildings(pPlayer)														-- Remove all gov buildings from pPlayer's capital
		
	-- Place building in capital
	local iNewGovBuildingType = GameInfoTypes[ g_tGovernment_Types[iSelectedGovernmentID][3] ]
	pCapitalCity:SetNumRealBuilding(iNewGovBuildingType, 1)							-- Adds new govt building building type from city	
	
	g_tSavedData.SetValue(string.format("GOV_OV_GOV_AGE_PLAYER_%02i", pPlayer:GetID()), Game.GetGameTurn())
	dPrint("Saving Gov Age. Player: " .. pPlayer:GetCivilizationShortDescription() .. "     GT: " .. Game.GetGameTurn() )
	
	local iGoldCost = GetGovernmentChangeCost(pPlayer)										-- Lower Culture and Gold by gov change costs	
	local iAnarchyDuration = PenalizGovernmentForChanging(pPlayer, iGoldCost)
	RemovePromotionFromOldGovernment(pPlayer, iSelectedGovernmentID)	

	PopulateLowerPanelforWellKnownCiv(pPlayer, g_tGovernment_Types[iSelectedGovernmentID][1], GOV_CD_NATIVE )
	IconHookup(iSelectedGovernmentID, 128, "SMAN_NATIONAL_GOVERNMENTS_ATLAS", Controls.CloseUpGovIcon);		-- Fill in the Gov Type on the LHS	
	Controls.CloseUpGovIcon:SetToolTipString(Locale.ConvertTextKey("TXT_KEY_GOV_OV_CLOSEUP_GOV_TT", g_tGovernment_Types[iSelectedGovernmentID][1], g_tGovernment_Types[iSelectedGovernmentID][4]))
		
	Controls.CloseUpChangeGovernment:SetDisabled(true)										-- Disable the launch button,so sel govt can only happen once a turn
	
	SentNotification(2, pPlayer:GetCivilizationShortDescription() .. " has changed its government from " .. g_tGovernment_Types[iStartingGovernmentID][1] .. " to " .. g_tGovernment_Types[iSelectedGovernmentID][1] .. "!",
				"The " .. pPlayer:GetCivilizationAdjective() .. " Empire has a new government!", pCapitalCity:GetX(), pCapitalCity:GetY() )

	if iAnarchyDuration > 0 then															-- If there is any anarchy, then...  (otherwise no anarchy imposed)
		SentNotification(1, "Due to their recent change of government, " .. pPlayer:GetCivilizationShortDescription() .. " is experiencing " .. iAnarchyDuration .. " turns of anarchy!",
				"The " .. pPlayer:GetCivilizationAdjective() .. " Empire is undergoing anarchy!", pCapitalCity:GetX(), pCapitalCity:GetY() )
	end
end


-- ========================================================================================================================
--		Change Government Popup - Main Function


function ChangeActivePlayerGovernment(iPlayerID)											-- Return a building ID or -1 for Random Building		
	g_ActivePlayer = Players[iPlayerID]
	local iGovernmentType = GetGovernmentType(g_ActivePlayer)
	g_StartingGovernmentID = iGovernmentType
	g_SelectedGovernmentID = iGovernmentType
	local iIdeology, sIdeology = GetIdeology(g_ActivePlayer)								-- Returns -1 for "No Ideology"
	
	dPrint("CAPG.  Civ: " .. g_ActivePlayer:GetCivilizationShortDescription() .. "     Gov: " .. g_tGovernment_Types[iGovernmentType][1] .. "     Gov Type: " .. iGovernmentType .. "     Ideology: " .. sIdeology .. "     Ideology ID: " .. iIdeology)
		
	Controls.GovOVSelGovMessage:SetToolTipString(Locale.ConvertTextKey("TXT_KEY_GOV_OV_SEL_GOV_MESSAGE_TT", g_tGovernment_Types[iGovernmentType][1] ))
	IconHookup(15, 80, "SMAN_NATIONAL_GOVERNMENTS_ATLAS", Controls.SelectGovernmentTopIcon)		-- Initial center icon will be "Unk"
	Controls.GovOVSelGovPopup:SetHide(false)												-- Display the Gov list popup
	UpdateGovernmentList(iGovernmentType, iIdeology, sIdeology)								-- Get a list of all non-wonder buildings in pTargetCity
end






